@@ -1,25 +1,24 @@
-import asyncio
-import sys
-from argparse import SUPPRESS, ArgumentParser, Namespace
-from ctypes import ArgumentError
+import asyncio, sys, re
+from io import BytesIO
+from typing import Any
 from pathlib import Path
 from pprint import pformat
-from typing import IO
 from urllib.parse import urlparse
 
 import keyring as secret_store
+from argparse import SUPPRESS, ArgumentParser, Namespace
 from aiohttp_client_cache.session import CachedSession
 from pytoml import loads as parse_toml
 
 from .structures import Intermediate, Resource
 
 
-def get_hash(file: Path | IO | bytes, hash_type: str = "sha256") -> str:
+def get_hash(file: Path | BytesIO | bytes, hash_type: str = "sha256") -> str:
 
     if isinstance(file, Path): data = file.read_bytes()
-    elif isinstance(file, IO): data = file.read()
+    elif isinstance(file, BytesIO): data = file.read()
     elif isinstance(file, bytes): data = file
-    else: raise ArgumentError("Incorrect file type!")
+    else: raise TypeError("Incorrect file type!")
         
     from hashlib import sha1, sha256, sha512
 
@@ -31,12 +30,12 @@ def get_hash(file: Path | IO | bytes, hash_type: str = "sha256") -> str:
         case "sha256": hash = sha256(data).hexdigest()
         case "sha512": hash = sha512(data).hexdigest()
         case "xxhash": hash = xxh3_64_hexdigest(data)
-        case "murmur2": hash = murmur2(bytes([b for b in data if b not in (9, 10, 13, 32)]), seed=1)
-        case _: raise ArgumentError("Incorrect hash type!")
+        case "murmur2": hash = murmur2(re.sub(rb"[\x09\x0A\x0D\x20]", b"", data), seed=1)
+        case _: raise TypeError("Incorrect hash type!")
 
     return str(hash)
 
-def get_hashes(file: Path | IO | bytes, *args: str):
+def get_hashes(file: Path | BytesIO | bytes, *args: str):
     return [get_hash(file, hash_type) for hash_type in args]
 
 async def add_github_token(session: CachedSession) -> None:
@@ -96,7 +95,6 @@ def parse_args() -> Namespace:
     arg_parser.add_argument('-o', '--output', dest='output', type=Path, default=Path.cwd())
     arg_parser.add_argument('--modrinth-search', dest='modrinth_search', type=str, choices=mr_search, default='exact')
     arg_parser.add_argument('--exclude-providers', dest='excluded_providers', type=str, nargs="+", choices=providers, default=str())
-    arg_parser.add_argument('--exclude-forbidden', dest='ignore_CF_flag', action='store_false')
     arg_parser.add_argument('--skip-cache', dest='skip_cache', action='store_true')
     arg_parser.add_argument('-v', '--version', dest='modpack_version', type=str)
 
@@ -126,12 +124,9 @@ def parse_args() -> Namespace:
 
     return args
 
-def read_config_into(cfg_path: Path, intermediate: Intermediate, exclude_forbidden: bool) -> None:
+def read_config_into(cfg_path: Path, intermediate: Intermediate) -> None:
 
-    forbidden_domains = ("edge.forgecdn.net", "media.forgecdn.net")
-    allowed_domains = ("cdn.modrinth.com", "edge.forgecdn.net", "media.forgecdn.net", "github.com", "raw.githubusercontent.com")
-    if exclude_forbidden: allowed_domains = tuple(domain for domain in allowed_domains if domain not in forbidden_domains)
-    
+    allowed_domains = ("cdn.modrinth.com", "edge.forgecdn.net", "media.forgecdn.net", "gitlab.com", "github.com", "raw.githubusercontent.com")
     lost_resources = [res for res in intermediate.resources if not res.providers]
 
     if cfg_path is not None and cfg_path.exists():
@@ -153,6 +148,9 @@ def read_config_into(cfg_path: Path, intermediate: Intermediate, exclude_forbidd
         resource = next((x for x in intermediate.resources if name == x.name or filename == x.file.name), None)
         if not resource: continue
 
+        if resource_config.get("optional") is True:
+            resource.optional = True
+
         match resource_config.get("action"):
 
             case None: # if action not specified, try to add a provider
@@ -199,4 +197,22 @@ async def resolve_conflicts(session: CachedSession, intermediate: Intermediate)
                 resource.file.hash.sha1 = sha1
                 resource.file.hash.sha256 = sha256 
                 resource.file.hash.sha512 = sha512 
-            
\ No newline at end of file
+
+
+import dataclasses, json
+class JsonEncoder(json.JSONEncoder):
+
+    def clean(self, value) -> dict | list | Any:
+        if isinstance(value, list): return [self.clean(x) for x in value if x]
+        elif isinstance(value, dict): return {key: self.clean(val) for key, val in value.items() if val}
+        else: return value
+
+    def default(self, o: Any) -> Any:
+        
+        if dataclasses.is_dataclass(o):
+            data = dataclasses.asdict(o)
+            return self.clean(data)
+        if isinstance(o, Path):
+            return o.as_posix()
+
+        return super().default(o)
