@@ -1,15 +1,15 @@
 from importlib import import_module
+from json import dump as write_json
 from pathlib import Path
 from shutil import rmtree
 
 from aiohttp import TCPConnector
 from aiohttp_client_cache.backends.filesystem import FileBackend
 from aiohttp_client_cache.session import CachedSession
-from json import dump as write_json
 
 from .Helpers.resourceAPI import ResourceAPI
-from .Helpers.utils import (add_github_token, parse_args, read_config_into,
-                            resolve_conflicts)
+from .Helpers.utils import (JsonEncoder, add_github_token, parse_args,
+                            read_config_into, resolve_conflicts)
 from .parser import Parser
 
 
@@ -19,9 +19,8 @@ async def run():
 
     ResourceAPI.modrinth_search_type = args.modrinth_search
     ResourceAPI.excluded_providers = args.excluded_providers
-    ResourceAPI.ignore_CF_flag = args.ignore_CF_flag
 
-    cache = FileBackend("mmc-export", use_temp=True, allowed_methods=("GET", "POST", "HEAD"))
+    cache = FileBackend("mmc-export", use_temp=True, urls_expire_after={'*.jar': -1}, allowed_methods=("GET", "POST", "HEAD"))
     async with CachedSession(cache=cache, connector=TCPConnector(limit=0)) as session: 
         if args.skip_cache: session.cache.disabled = True # type: ignore
 
@@ -42,14 +41,14 @@ async def run():
         intermediate = await parser.parse()
         
         if version := args.modpack_version: intermediate.version = version
-        read_config_into(args.config, intermediate, not args.ignore_CF_flag)
+        read_config_into(args.config, intermediate)
         await resolve_conflicts(session, intermediate) # type: ignore
 
         for format in args.formats:
 
             if format == "Intermediate":
                 with open(args.output / "intermediate_output.json", "w") as file:
-                    write_json(intermediate.to_dict(), file, indent=4)
+                    write_json(intermediate, file, indent=4, cls=JsonEncoder)
                 continue
 
             module = import_module(f".Formats.{format.lower()}", "mmc_export")
